
import java.awt.Dimension;
import java.awt.GridLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * MapPlane es un contenedor de {@link Tileset.Tile}, las instancias generadas por 
 * esta clase corresponden a planos utilizados por {@link MapContainer} para 
 * generar la vista por capas del mapa desplegado al usuario.
 * @author Jaime
 */

public class MapPlane{
   private final Tileset.Tile[][] MapTiles;
   private final Tileset MapTileset;
   private final MapView view;
   private final int width;
   private final int height;
   
    /**
     * Genera un nuevo MapPlane con el {@link Tileset} y las dimensiones 
     * especificadas.
     * @param width Ancho en numero de {@link Tileset.Tile} del plano.
     * @param height Alto en numero de {@link Tileset.Tile} del plano.
     * @param MapTileset {@link Tileset} a utilizar por el plano.
     */
    public MapPlane(int width, int height, Tileset MapTileset){
      this.width = width;
      this.height = height;
      this.MapTileset = MapTileset; // = new Tileset("./image/Tileset.png");
      MapTiles = new Tileset.Tile[height][width];
      for(int r = 0; r<height; r++){
         for(int c = 0; c<width; c++){
            MapTiles[r][c] = this.MapTileset.getEmptyTile();
         }
      }
      view = new MapView(width,height);
   }
   
    /**
     * Llena el plano con copias del {@link Tileset.Tile} seleccionadas, 
     * referidas a la esquina superior izquierda del plano y el arreglo de 
     * {@link Tileset.Tile} en cuestion. Si no hay {@link Tileset.Tile} 
     * seleccionados en el {@link Tileset} activo, ninguna accion es realizada. 
     */
    public void FloodMap(){
      Tileset.Tile[][] SelectedTiles = MapTileset.getSelectedTiles();
      if(SelectedTiles==null)
         return;
      
      for(int r = 0; r<height; r++){
         for(int c = 0; c<width; c++){
            MapTiles[r][c] = SelectedTiles[r%(SelectedTiles.length)][c%(SelectedTiles[0].length)];
         }
      }
   }

    /**
     * Retorna la vista del mapa.
     * @return {@link MapPlane.MapView} con la vista del plano.
     */
    public MapView getMapView(){
      return view;
   }
   
    /**
     * Entrega la vista del {@link Tileset} asociado al objeto.
     * @return JScrollPane con la vista del Tileset asociado al plano.
     */
    public JScrollPane getTilesetView(){
      return MapTileset.getView();
   }

    private void setNewMapTiles(int x_offset, int y_offset){
      Tileset.Tile[][] SelectedTiles = MapTileset.getSelectedTiles();
      if(SelectedTiles==null)
         return;
      
      for(int r=y_offset; r<Math.min(SelectedTiles.length+y_offset,height); r++){
         for(int c=x_offset; c<Math.min(SelectedTiles[0].length+x_offset,width); c++){
            MapTiles[r][c] = SelectedTiles[r-y_offset][c-x_offset];
         }
      }
      view.redraw();
   }
   
    /**
     * Vista para la clase {@link MapPlane}, provee el unico metodo 
     * {@link #setNewMapTiles(int, int)} para cambiar el estado del objeto 
     * {@link MapPlane} asociado.
     */
    public final class MapView extends JPanel {
      private MapView(int width, int height){
         setLayout(new GridLayout(height,width));
         redraw();
      }
      
      private void redraw(){
         this.removeAll();
         for(int r = 0; r<height; r++){
            for(int c = 0; c<width; c++){
               add(MapTiles[r][c].getView());
            }
         }
         this.revalidate();
         this.repaint();
         Dimension Final_size = new Dimension(Tileset.TILE_PIX_SIZE*width,Tileset.TILE_PIX_SIZE*height);
         this.setMaximumSize(Final_size);
         this.setPreferredSize(Final_size);
         this.setOpaque(false);
      }
      
          
        /**
         * Coloca los {@link Tileset.Tile} seleccionados en la posicion del plano 
         * deseada, referidas a la esquina superior izquierda del plano 
         * (x_offset=0, y_offset=0) y el arreglo de {@link Tileset.Tile} en 
         * cuestion.
         * @param x_offset Coordenada horizontal de la posicion del 
         * {@link Tileset.Tile} a colocar
         * @param y_offset Coordenada vertical de la posicion del 
         * {@link Tileset.Tile} a colocar
         */
       public void setNewMapTiles(int x_offset, int y_offset){
         MapPlane.this.setNewMapTiles(x_offset, y_offset);
      }
   }
}