
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSpinner.DefaultEditor;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 * Esta clase se encarga de contener al contexto del juego creado. Almacena 
 * listas con {@link MapContainer} y {@link Tileset}, ademas del nombre del
 * proyecto.
 * @author Jaime
 */
public class Project {
   private ArrayList<MapContainer> RootMaps;
   private ArrayList<Tileset> AvailableTilesets;
   private String name;
   private ProjectView view;
   
    /**
     * Crea un nuevo proyecto con los Tileset disponibles.
     * @param name Nombre del proyecto.
     * @param AvailableTilesets Lista de Tileset disponibles en el programa.
     * @param PanelListener Listener que cambia las vistas activas segun lo 
     * dicta la vista de la instancia de Project.
     */
    public Project(String name, ArrayList<Tileset> AvailableTilesets, TreeSelectionListener PanelListener){
      this.name = name;
      this.AvailableTilesets = AvailableTilesets;
      RootMaps = new ArrayList<MapContainer>();
      RootMaps.add(new MapContainer(20,15,AvailableTilesets.get(0)));
      view = new ProjectView(PanelListener);
   }
   
    /**
     * Llena la capa superior del mapa mostrado en la vista activa con los 
     * Tile seleccionados.
     */
    public void FloodLayer(){
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           view.tree.getLastSelectedPathComponent();
      
      if(node==null) return;
      
      Object nodeInfo = node.getUserObject();
      
      MapContainer SelectedMap = (MapContainer)nodeInfo;
      
      SelectedMap.FloodLayer();
   }
   
    /**
     * Registra cual capa es la superior en la vista activa.
     * @param layer Numero de la capa a subir.
     */
    public void setTopLayer(int layer){
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           view.tree.getLastSelectedPathComponent();
      
      if(node==null) return;
      
      Object nodeInfo = node.getUserObject();
      
      MapContainer SelectedMap = (MapContainer)nodeInfo;
      
      SelectedMap.setTopMapLayer(layer);
   }
   
    /**
     * Retorna la vista del Projecto, la cual consiste en un arbol con los 
     * distintos mapas, al hacer click derecho sobre un mapa se puede agregar o 
     * quitar mas mapas
     * @return JPanel con la vista del proyecto.
     */
    public JPanel getView(){
      return view;
   }
   
    /**
     * Retorna la vista del mapa seleccionado.
     * @return JLayeredPane con la vista del MapContainer seleccionado.
     */
    public JLayeredPane getFocusedMapView(){
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           view.tree.getLastSelectedPathComponent();
      Object nodeInfo = node.getUserObject();
      
      MapContainer SelectedMap = (MapContainer)nodeInfo;
      return SelectedMap.getView();
      
   }
   
    /**
     * Retorna la vista del Tileset activo, segun el MapContainer seleccionado.
     * @return JScrollPane con la vista del Tileset activo.
     */
    public JScrollPane getFocusedTilesetView(){
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           view.tree.getLastSelectedPathComponent();
      Object nodeInfo = node.getUserObject();
      
      MapContainer SelectedMap = (MapContainer)nodeInfo;
      return SelectedMap.getTilesetView();
   }
   
    private boolean removeMapContainer(int ContainerIndex){
      boolean result = false;
      for(int i=0; i<RootMaps.size(); i++){
         if(RootMaps.get(i).getContainerID() == ContainerIndex){
            RootMaps.remove(i);
            result = true;
         }
      }
      return result;
   }
   
    private MapContainer addMapContainer(int width, int height, Tileset MapTileset){
      if(RootMaps==null)
         RootMaps = new ArrayList<MapContainer>();
      
      MapContainer newMap = new MapContainer(width, height, MapTileset);
      RootMaps.add(newMap);
      return newMap;
   }
   
    private MapContainer addMapContainer(int width, int height, Tileset MapTileset, String name){
      if(RootMaps==null)
         RootMaps = new ArrayList<MapContainer>();   
   
      MapContainer newMap = new MapContainer(width, height, MapTileset, name);
      RootMaps.add(newMap);
      return newMap;
   }  
   
   final class ProjectView extends JPanel implements MouseListener{
      
      private final JTree tree;
      private final DefaultMutableTreeNode RootNode;
      private final DefaultTreeModel treeModel;
      
      ProjectView(TreeSelectionListener PanelListener){
         super(new GridLayout(1,0));

         //Create the nodes.
         RootNode = new DefaultMutableTreeNode(name);
         
         createNodes(RootNode, RootMaps);
         
         treeModel = new DefaultTreeModel(RootNode);

         //Create a tree that allows one selection at a time.
         tree = new JTree(treeModel);
         tree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);
         tree.setShowsRootHandles(true);
         //tree.setRootVisible(false);
         
         //Listen for when the selection changes.
         tree.addTreeSelectionListener(PanelListener);
         
         //Create the scroll pane and add the tree to it. 
         JScrollPane treeView = new JScrollPane(tree);
         treeView.setMinimumSize(new Dimension(100,100));
         
         tree.addMouseListener(this);
         
         add(treeView);
      }
   
      private void createNodes(DefaultMutableTreeNode currentTop, ArrayList<MapContainer> currentMap) {
         if(currentMap == null) return;
         
         DefaultMutableTreeNode currentNode = null;
         
         for(int i = 0; i<currentMap.size(); i++){
            currentNode = new DefaultMutableTreeNode(currentMap.get(i));
            createNodes(currentNode,currentMap.get(i).getChildren());
            currentTop.add(currentNode);
         }
      }
      
      private void doPop(MouseEvent e, boolean allowAdd){
         PopUpDemo menu = new PopUpDemo(allowAdd);
         menu.show(e.getComponent(), e.getX(), e.getY());
      }
      
      public void mouseClicked(MouseEvent e){}
      public void mousePressed(MouseEvent e){}
      public void mouseReleased(MouseEvent e){
         if (SwingUtilities.isRightMouseButton(e)) {

         int selRow = tree.getRowForLocation(e.getX(), e.getY());
         if(selRow != -1) {
            tree.setSelectionRow(selRow);
             if(selRow == 0) {
                 doPop(e, false);
                 System.out.println(selRow);
             }
             else {
                 doPop(e,true);
             }
         }
     
         }
      }
      public void mouseEntered(MouseEvent e){}
      public void mouseExited(MouseEvent e){}

      private final class PopUpDemo extends JPopupMenu implements ActionListener {
         JMenuItem AddMapItem;
         JMenuItem DeleteMapItem;
         public PopUpDemo(boolean allowAdd){
            AddMapItem = new JMenuItem("New Map...");
            AddMapItem.addActionListener(this);
            add(AddMapItem);
            
            DeleteMapItem = new JMenuItem("Delete.");
            DeleteMapItem.addActionListener(this);
            DeleteMapItem.setEnabled(allowAdd);
            add(DeleteMapItem);
         }
         
         public void actionPerformed(ActionEvent e){
            if(e.getSource() == AddMapItem)
               addMapContainer();
            if(e.getSource()== DeleteMapItem)
               removeMapContainer();
            
         }
         
        private void addMapContainer(){ 
           DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                 tree.getLastSelectedPathComponent();
           Object nodeInfo = node.getUserObject();
           MapContainer newMap;

           JTextField MapName = new JTextField(MapContainer.getNextContainerName());
           SpinnerNumberModel WidthSpinnerModel  =  new SpinnerNumberModel(20, 20, 500, 1);
           SpinnerNumberModel HeightSpinnerModel =  new SpinnerNumberModel(15, 15, 500, 1);
           JSpinner WidthSpinner  = new JSpinner(WidthSpinnerModel);
           ((DefaultEditor) WidthSpinner.getEditor()).getTextField().setEditable(false);
           JSpinner HeightSpinner = new JSpinner(HeightSpinnerModel);
           ((DefaultEditor) HeightSpinner.getEditor()).getTextField().setEditable(false);
           JComboBox TilesetComboBox = new JComboBox(AvailableTilesets.toArray());

           final JComponent[] inputs = new JComponent[] {
                   new JLabel("Map Name:"),
                   MapName,
                   new JLabel("Width"),
                   WidthSpinner,
                   new JLabel("Height"),
                   HeightSpinner,
                   new JLabel("Tileset"),
                   TilesetComboBox
           };
           int result = JOptionPane.showConfirmDialog(null, inputs, "Set New Map Properties", JOptionPane.PLAIN_MESSAGE);
           if (result == JOptionPane.OK_OPTION) {
              if(nodeInfo instanceof MapContainer){
                 MapContainer SelectedMap = (MapContainer)nodeInfo;

              if("".equals(MapName.getText()))
                 newMap = SelectedMap.addMapContainer(WidthSpinnerModel.getNumber().intValue(), HeightSpinnerModel.getNumber().intValue(), (Tileset)TilesetComboBox.getSelectedItem());
              else
                 newMap = SelectedMap.addMapContainer(WidthSpinnerModel.getNumber().intValue(), HeightSpinnerModel.getNumber().intValue(), (Tileset)TilesetComboBox.getSelectedItem(),MapName.getText());

              DefaultMutableTreeNode parentNode = null;
              TreePath parentPath = tree.getSelectionPath();

              if (parentPath == null) {
                 parentNode = RootNode;
              } else {
                  parentNode = (DefaultMutableTreeNode)
                               (parentPath.getLastPathComponent());
              }

              DefaultMutableTreeNode childNode = 
                        new DefaultMutableTreeNode(newMap);
              treeModel.insertNodeInto(childNode, parentNode, parentNode.getChildCount());
              tree.scrollPathToVisible(new TreePath(childNode.getPath()));
              } else {
              if("".equals(MapName.getText()))
                 newMap = Project.this.addMapContainer(WidthSpinnerModel.getNumber().intValue(), HeightSpinnerModel.getNumber().intValue(), (Tileset)TilesetComboBox.getSelectedItem());
              else
                 newMap = Project.this.addMapContainer(WidthSpinnerModel.getNumber().intValue(), HeightSpinnerModel.getNumber().intValue(), (Tileset)TilesetComboBox.getSelectedItem(),MapName.getText());

              DefaultMutableTreeNode parentNode = null;
              TreePath parentPath = tree.getSelectionPath();

              if (parentPath == null) {
                 parentNode = RootNode;
              } else {
                  parentNode = (DefaultMutableTreeNode)
                               (parentPath.getLastPathComponent());
              }

              DefaultMutableTreeNode childNode = 
                        new DefaultMutableTreeNode(newMap);
              treeModel.insertNodeInto(childNode, parentNode, parentNode.getChildCount());
              tree.scrollPathToVisible(new TreePath(childNode.getPath()));

              }
           }
        }
        private void removeMapContainer(){
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                           tree.getLastSelectedPathComponent();
            Object nodeInfo = node.getUserObject();

            MapContainer SelectedMap = (MapContainer)nodeInfo;

            if(((DefaultMutableTreeNode)node.getParent()).getUserObject() instanceof MapContainer){
                MapContainer MapParent = (MapContainer)((DefaultMutableTreeNode)node.getParent()).getUserObject();

                System.out.println(MapParent.removeMapContainer(SelectedMap.getContainerID()));
                treeModel.removeNodeFromParent(node);
            } else {
                Project.this.removeMapContainer(SelectedMap.getContainerID());   
                treeModel.removeNodeFromParent(node);
            }
        }
        
      }
      
     
   }
}
