
import java.awt.Color;
import java.awt.image.BufferedImage;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.*;

/**
 * Contenedor de {@link Tileset.Tile}, los cuales son agrupados en un contexto 
 * comun segun el recurso grafico que tienen asociado. Por convencion se 
 * considera el Tile de la esquina superior izquierda (ID=0) como un Tile vacio.
 * @author Jaime
 */
public class Tileset{

   private BufferedImage BaseTilesetImage;
   private final Tile TileArray[][];
   private Tile SelectedTileArray[][] = null;
   private int rows;
   private int columns;
   private final String name;
   private JScrollPane view;
   
    /**
     * Dimension en pixeles del lado de un {@link Tileset.Tile} 
     */
    public static int TILE_PIX_SIZE = 32;

    /**
     * Crea un nuevo Tileset a partir de la imagen encontrada en 
     * <i>imagePath</i> con el nombre <i>name</i>.
     * @param name Nombre del nuevo Tileset.
     * @param imagePath Ruta de la imagen a utilizar en el Tileset.
     */
    public Tileset(String name, String imagePath){
      // Load main tileset *.png in RGBA mode
      File img;
      BufferedImage image;
      try {
         img = new File(imagePath);
         image = ImageIO.read(img); 
         BaseTilesetImage = new BufferedImage(
         image.getWidth(), image.getHeight(), BufferedImage.TYPE_INT_ARGB);
         BaseTilesetImage.getGraphics().drawImage(image,0,0,null);
         rows = image.getHeight()/TILE_PIX_SIZE;
         columns = image.getWidth()/TILE_PIX_SIZE;
      } catch (IOException e) { 
         e.printStackTrace(); 
      }
            
      // Parse individual tiles in tileset
      TileArray = new Tile[rows][columns];
      for(int r = 0; r<rows; r++){
         for(int c = 0; c<columns; c++){
            TileArray[r][c] = new Tile(BaseTilesetImage.getSubimage(c*TILE_PIX_SIZE,r*TILE_PIX_SIZE,TILE_PIX_SIZE,TILE_PIX_SIZE),columns*c+r);
         }
      }
      this.name = name;
      view = new JScrollPane(new TilesetViewport(imagePath, this));
      view.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
      
   }
    
    /**
     * Retorna las dimensiones del Tileset.
     * @return Dimension el alto en filas y el largo en columnas
     */
    public Dimension getTilesetDimension(){
        return new Dimension(columns,rows);
    }
   
   @Override
   public String toString(){
      return name;
   }
      
    /**
     * Asigna las propiedades del {@link Tileset.Tile} en la posicion 
     * {<i>row</i>,<i>col</i>}.
     * @param row Fila del {@link Tileset.Tile} a asignar.
     * @param col Columna del {@link Tileset.Tile} a asignar.
     * @param pass Propiedad caminable del Tile.
     * @param UP Propiedad caminable por arriba del Tile.
     * @param DOWN Propiedad caminable por abajo del Tile.
     * @param LEFT Propiedad caminable por la izquierda del Tile.
     * @param RIGHT Propiedad caminable por la derecha del Tile.
     * @param prior Prioridad del Tile.
     */
    public void setTile(int row, int col,boolean pass, boolean UP,boolean DOWN,boolean LEFT,boolean RIGHT, int prior){
       TileArray[row][col].setParam(pass,UP,DOWN,LEFT,RIGHT,prior);
   }   

    /**
     * Retorna la vista del Tileset.
     * @return JScrollPane con la vista del Tileset.
     */
    public JScrollPane getView(){
      return view;
   }
   
    /**
     * Registra los Tiles seleccionados segun las coordenadas entregadas 
     * siguiendo una estructura rectangular. En caso de entregar indices fuera 
     * de rango no se realiza ninguna accion.
     * @param x_i Coordenada horizontal inicial del elemento seleccionado.
     * @param x_f Coordenada horizontal final del elemento seleccionado.
     * @param y_i Coordenada vertical inicial del elemento seleccionado.
     * @param y_f Coordenada vertical final del elemento seleccionado.
     */
    public void setSelectedTiles(int x_i, int x_f, int y_i, int y_f){     
        if(x_i<0||x_i>=columns) return;
        if(x_f<0||x_f>=columns) return;
        if(y_i<0||y_i>=rows) return;
        if(y_f<0||y_f>=rows) return;
      SelectedTileArray = new Tile[y_f-y_i+1][x_f-x_i+1];
      
      for (int y = 0; y < SelectedTileArray.length; y++) {
         for (int x = 0; x < SelectedTileArray[0].length; x++) {
            SelectedTileArray[y][x] = TileArray[y_i+y][x_i+x];
         }
      }     
   }
   
    /**
     * Entrega un arreglo con los Tile seleccionados.
     * @return Tile[][] con los Tile seleccionados.
     */
    public Tile[][] getSelectedTiles(){
      return SelectedTileArray;
   }
   
    /**
     * Entrega el Tile de la esquina superior izquierda, considerado como el 
     * Tile vacio.
     * @return Tile vacio.
     */
    public Tile getEmptyTile(){
      return TileArray[0][0];
   }

    /**
     * Elemento basico del escenario del juego, contenido en un Tileset y 
     * desplegado en {@link MapPlane}, esta clase tiene las propiedades de 
     * cada cuadrado de escenario y su imagen.
     */
    public class Tile {
      private final BufferedImage TileImage;
      private final int TileIndex;
      private boolean passage;
      private boolean[] directions=new boolean[4]; //[UP,DOWN,LEFT,RIGHT]
      private int priority;
      
       /**
        * Crea un nuevo Tile con la imagen <i>TileImage</i> y el indice 
        * <i>TileIndex</i>
        * @param TileImage Imagen del nuevo Tile.
        * @param TileIndex ID del nuevo Tile.
        */
       public Tile(BufferedImage TileImage, int TileIndex){
         this.TileImage = TileImage;
         this.TileIndex = TileIndex;
         this.passage=true;
         for(int i=0; i<4;i++){
             directions[i]=true;
         }
         this.priority=4;
         //this.configview=configview;
      }

       private void setParam(boolean pass, boolean UP,boolean DOWN,boolean LEFT,boolean RIGHT, int prior){
          this.passage=pass;
          this.priority=prior;
          directions[0]=UP;
          directions[1]=DOWN;
          directions[2]=LEFT;
          directions[3]=RIGHT;
      }
           
       /**
        * Fabrica vistas para el Tile correspondiente. Si se llama mas de una 
        * vez este metodo para un mismo Tile se generara una nueva vista para 
        * el Tile distinta a la de la ultima llamada.
        * @return JLabel con el icono del Tile.
        */
       public JLabel getView(){
         JLabel view = new JLabel(new ImageIcon(TileImage));
         view.setBackground(new Color(0, 0, 255, 50));
         if(TileIndex==0){
            view.setOpaque(true);
         } else {
            view.setOpaque(false);
         }
         return view;
      }
   }
   
   private class TilesetViewport extends JLayeredPane implements Scrollable {

      public TilesetViewport(String Tileset_image_path, Tileset driver_class){
         SpringLayout layout = new SpringLayout();
         this.setLayout(layout);

         ImageIcon MyTilesetIcon = new ImageIcon(getClass().getResource(Tileset_image_path));
         JLabel BG_image = new JLabel(MyTilesetIcon);

         this.add(BG_image,JLayeredPane.DEFAULT_LAYER);         
         layout.putConstraint(SpringLayout.WEST, BG_image,
                     0,
                     SpringLayout.WEST, this);
         layout.putConstraint(SpringLayout.NORTH, BG_image,
                     0,
                     SpringLayout.NORTH, this);

         BG_image.setPreferredSize(new Dimension(MyTilesetIcon.getIconWidth(),MyTilesetIcon.getIconHeight()));

         JPanel SelectableFrontPanel = new SelectableGrid(MyTilesetIcon.getIconHeight(), driver_class);

         this.add(SelectableFrontPanel,JLayeredPane.PALETTE_LAYER);
         layout.putConstraint(SpringLayout.WEST, SelectableFrontPanel,
                     0,
                     SpringLayout.WEST, this);
         layout.putConstraint(SpringLayout.NORTH, SelectableFrontPanel,
                     0,
                     SpringLayout.NORTH, this);
         SelectableFrontPanel.setPreferredSize(new Dimension(MyTilesetIcon.getIconWidth(),MyTilesetIcon.getIconHeight()));

         this.setPreferredSize(new Dimension(MyTilesetIcon.getIconWidth(),MyTilesetIcon.getIconHeight()));
      }

      @Override
      public Dimension getPreferredScrollableViewportSize(){
         return this.getPreferredSize();//new Dimension(200, 200);
      }

      @Override
      public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction){
         return TILE_PIX_SIZE;
      }

      @Override
      public boolean getScrollableTracksViewportHeight(){
         return false;
      }

      @Override
      public boolean getScrollableTracksViewportWidth(){
         return true;
      }

      @Override
      public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction){
         return TILE_PIX_SIZE;
      }
   }

}